﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace TestLibrary
{
    /// <summary>
    /// 데이터 페이지
    /// </summary>
    /// <typeparam name="TItem">항목 타입</typeparam>
    public class DataPage<TItem> where TItem : class
    {
        #region 항목 리스트 - ItemList

        /// <summary>
        /// 항목 리스트
        /// </summary>
        public IList<DataWrapper<TItem>> ItemList { get; private set; }

        #endregion
        #region 최근 사용 시간 - LastUsedTime

        /// <summary>
        /// 최근 사용 시간
        /// </summary>
        public DateTime LastUsedTime { get; set; }

        #endregion
        #region 사용 여부 - InUse

        /// <summary>
        /// 사용 여부
        /// </summary>
        public bool InUse
        {
            get
            {
                return ItemList.Any(wrapper => wrapper.InUse);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DataPage(firstIndex, pageLength)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="firstIndex">첫번째 인덱스</param>
        /// <param name="pageLength">페이지 길이</param>
        public DataPage(int firstIndex, int pageLength)
        {
            ItemList = new List<DataWrapper<TItem>>(pageLength);

            for(int i = 0; i < pageLength; i++)
            {
                ItemList.Add(new DataWrapper<TItem>(firstIndex + i));
            }

            LastUsedTime = DateTime.Now;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 만들기 - Populate(itemList)

        /// <summary>
        /// 만들기
        /// </summary>
        /// <param name="itemList">항목 리스트</param>
        public void Populate(IList<TItem> itemList)
        {
            int index = 0;
 
            int i;

            for(i = 0; i < itemList.Count && i < ItemList.Count; i++)
            {
                ItemList[i].Item = itemList[i];

                index = ItemList[i].Index;
            }

            while(i < itemList.Count)
            {
                index++;

                ItemList.Add
                (
                    new DataWrapper<TItem>(index)
                    {
                        Item = itemList[i]
                    }
                );

                i++;
            }

            while(i < ItemList.Count)
            {
                ItemList.RemoveAt(ItemList.Count - 1);
            }
        }

        #endregion
    }
}