﻿using System.ComponentModel;

namespace TestLibrary
{
    /// <summary>
    /// 데이터 래퍼
    /// </summary>
    /// <typeparam name="TItem">항목 타입</typeparam>
    public class DataWrapper<TItem> : INotifyPropertyChanged where TItem : class
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - 속성 변경시

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 인덱스
        /// </summary>
        private int index;

        /// <summary>
        /// 데이터
        /// </summary>
        private TItem item;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 인덱스 - Index

        /// <summary>
        /// 인덱스
        /// </summary>
        public int Index
        {
            get
            {
                return this.index;
            }
        }

        #endregion
        #region 항목 번호 - ItemNumber

        /// <summary>
        /// 항목 번호
        /// </summary>
        public int ItemNumber
        {
            get
            {
                return this.index + 1;
            }
        }

        #endregion
        #region 로딩 여부 - IsLoading

        /// <summary>
        /// 로딩 여부
        /// </summary>
        public bool IsLoading
        {
            get
            {
                return this.item == null;
            }
        }

        #endregion
        #region 항목 - Item

        /// <summary>
        /// 항목
        /// </summary>
        public TItem Item
        {
            get
            {
                return this.item;
            }
            internal set
            {
                this.item = value;

                this.FirePropertyChangedEvent("Item");
                this.FirePropertyChangedEvent("IsLoading");
            }
        }

        #endregion
        #region 사용 여부 - InUse

        /// <summary>
        /// 사용 여부
        /// </summary>
        public bool InUse
        {
            get
            {
                return PropertyChanged != null;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DataWrapper(index)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="index">인덱스</param>
        public DataWrapper(int index)
        {
            this.index = index;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        private void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}