﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace TestProject
{
    /// <summary>
    /// 일자 범위 선택기
    /// </summary>
    public partial class DateRangePicker : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작일 변경시 - FromDateChanged

        /// <summary>
        /// 시작일 변경시
        /// </summary>
        public event EventHandler FromDateChanged;

        #endregion
        #region 종료일 변경시 - ToDateChanged

        /// <summary>
        /// 종료일 변경시
        /// </summary>
        public event EventHandler ToDateChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 시작일 속성 - FromDateProperty

        /// <summary>
        /// 시작일 속성
        /// </summary>
        public static readonly DependencyProperty FromDateProperty = DependencyProperty.Register
        (
            "FromDate",
            typeof(Nullable<DateTime>),
            typeof(DateRangePicker),
            new PropertyMetadata(FromDatePropertyChangedCallback)
        );

        #endregion
        #region 종료일 속성 - ToDateProperty

        /// <summary>
        /// 종료일 속성
        /// </summary>
        public static readonly DependencyProperty ToDateProperty = DependencyProperty.Register
        (
            "DateTo",
            typeof(Nullable<DateTime>),
            typeof(DateRangePicker),
            new PropertyMetadata(ToDatePropertyChangedCallback)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작일 - FromDate

        /// <summary>
        /// 시작일
        /// </summary>
        public Nullable<DateTime> FromDate
        {
            get
            {
                return (Nullable<DateTime>)GetValue(FromDateProperty);
            }
            set
            {
                SetValue(FromDateProperty, value);
            }
        }

        #endregion
        #region 종료일 - ToDate

        /// <summary>
        /// 종료일
        /// </summary>
        public Nullable<DateTime> ToDate
        {
            get
            {
                return (Nullable<DateTime>)GetValue(ToDateProperty);
            }
            set
            {
                SetValue(ToDateProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DateRangePicker()

        /// <summary>
        /// 생성자
        /// </summary>
        public DateRangePicker()
        {
            InitializeComponent();

            this.fromDatePicker.BlackoutDates.Add
            (
                new CalendarDateRange
                (
                    DateTime.Today.AddDays(1),
                    DateTime.MaxValue
                )
            );

            this.fromDatePicker.SetBinding
            (
                DatePicker.SelectedDateProperty,
                new Binding("FromDate")
                {
                    Source = this,
                    Mode   = BindingMode.TwoWay
                }
            );

            this.toDatePicker.BlackoutDates.Add
            (
                new CalendarDateRange
                (
                    DateTime.Today.AddDays(1),
                    DateTime.MaxValue
                )
            );

            this.toDatePicker.SetBinding
            (
                DatePicker.SelectedDateProperty,
                new Binding("ToDate")
                {
                    Source = this,
                    Mode   = BindingMode.TwoWay
                }
            );
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 시작일 속성 변경시 콜백 처리하기 - FromDatePropertyChangedCallback(d, e)

        /// <summary>
        /// 시작일 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void FromDatePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DateRangePicker dateRangePicker = (DateRangePicker)d;

            dateRangePicker.ProcessFromDateChanged();
        }

        #endregion
        #region 종료일 속성 변경시 콜백 처리하기 - ToDatePropertyChangedCallback(d, e)

        /// <summary>
        /// 종료일 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void ToDatePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DateRangePicker dateRangePicker = (DateRangePicker)d;

            dateRangePicker.FireDateChancedEvent(dateRangePicker.ToDateChanged);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 시작일 변경시 처리하기 - ProcessFromDateChanged()

        /// <summary>
        /// 시작일 변경시 처리하기
        /// </summary>
        private void ProcessFromDateChanged()
        {
            this.toDatePicker.BlackoutDates.Clear();

            if(FromDate.HasValue)
            {
                DateTime fromDate = FromDate.Value;

                if(ToDate.HasValue)
                {
                    DateTime toDate = ToDate.Value;

                    if(toDate <= fromDate)
                    {
                        ToDate = null;
                    }
                }

                this.toDatePicker.BlackoutDates.Add
                (
                    new CalendarDateRange
                    (
                        DateTime.MinValue,
                        fromDate
                    )
                );
            }

            this.toDatePicker.BlackoutDates.Add
            (
                new CalendarDateRange
                (
                    DateTime.Today.AddDays(1),
                    DateTime.MaxValue
                )
            );

            this.FireDateChancedEvent(FromDateChanged);
        }

        #endregion
        #region 일자 변경 이벤트 발생시키기 - FireDateChancedEvent(handler)

        /// <summary>
        /// 일자 변경 이벤트 발생시키기
        /// </summary>
        /// <param name="handler">이벤트 핸들러</param>
        private void FireDateChancedEvent(EventHandler handler)
        {
            handler?.Invoke(this, EventArgs.Empty);
        }

        #endregion
    }
}