﻿using System;
using System.Collections.Generic;
using System.Linq;

using TestLibrary;

namespace TestProject
{
    /// <summary>
    /// 고객 공급자
    /// </summary>
    public class CustomerProvider : IItemProvider<Customer>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 고객 리스트
        /// </summary>
        private readonly List<Customer> customerList;

        /// <summary>
        /// 시작일
        /// </summary>
        private readonly DateTime? startDate;

        /// <summary>
        /// 종료일
        /// </summary>
        private readonly DateTime? endDate;

        /// <summary>
        /// 정렬 필드
        /// </summary>
        private readonly string sortField;

        /// <summary>
        /// 카운트
        /// </summary>
        private int count;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomerProvider()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomerProvider()
        {
            this.startDate = DateTime.Today.AddYears(-100);
            this.endDate   = DateTime.Today.AddYears(100);
            this.sortField = "CustomerSince DESC";

            this.customerList = new List<Customer>();

            for(int i = 0; i < 1000000; i++)
            {
                this.customerList.Add
                (
                    new Customer
                    {
                        AmountPaidInternationalCalls = i % 100,
                        AmountPaidLocalCalls         = i % 100,
                        AmountPaidNationalCalls      = i % 100,
                        CustomerSince                = this.startDate.Value.AddDays(i),
                        FirstName                    = string.Format("Customer {0}", i),
                        Id                           = i,
                        LastName                     = string.Format("LastName {0}", i),
                        JoinedPreferredProgram       = i % 2 == 0,
                        NumberFamilyMembersInPlan    = i % 4,
                        Region                       = i % 100,
                    }
                );
            }
        }

        #endregion
        #region 생성자 - CustomerProvider(startDate, endDate, sortField)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="startDate">시작일</param>
        /// <param name="endDate">종료일</param>
        /// <param name="sortField">정렬 필드</param>
        public CustomerProvider(DateTime? startDate, DateTime? endDate, string sortField) : this()
        {
            if(startDate != null)
            {
                this.startDate = startDate;
            }

            if(endDate != null)
            {
                this.endDate = endDate;
            }

            this.sortField = sortField;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 카운트 가져오기 - FetchCount()

        /// <summary>
        /// 카운트 가져오기
        /// </summary>
        /// <returns>카운트</returns>
        public int FetchCount()
        {
            this.count = this.customerList.Count(e => e.CustomerSince >= this.startDate && e.CustomerSince <= this.endDate);

            return this.count;
        }

        #endregion
        #region 범위 가져오기 - FetchRange(startIndex, pageCount, overallCount)

        /// <summary>
        /// 범위 가져오기
        /// </summary>
        /// <param name="startIndex">시작 인덱스</param>
        /// <param name="pageCount">페이지 카운트</param>
        /// <param name="overallCount">전체 카운트</param>
        /// <returns>고객 리스트</returns>
        public IList<Customer> FetchRange(int startIndex, int pageCount, out int overallCount)
        {
            // 이 경우 데이터베이스의 데이터가 변경되지 않는다고 가정하기 때문에 카운트를 다시 가져오지 않아도 된다.
            overallCount = this.count;

            if(this.sortField.Contains("DESC"))
            {
                return this.customerList.Where(e => e.CustomerSince >= this.startDate && e.CustomerSince <= this.endDate)
                                        .OrderBy(e => e.FirstName)
                                        .Skip(startIndex)
                                        .Take(pageCount).ToList();
            }
            else
            {
                return this.customerList.Where(e => e.CustomerSince >= this.startDate && e.CustomerSince <= this.endDate)
                                        .OrderByDescending(e => e.FirstName)
                                        .Skip(startIndex)
                                        .Take(pageCount).ToList();
            }
        }

        #endregion
    }
}